@@ -67,6 +67,7 @@ pub struct TemplateState {
     tx_id: u64,
     transactions: Vec<TemplateTransaction>,
     request_gap: bool,
+    #[allow(dead_code)]
     response_gap: bool,
 }
 
@@ -316,7 +317,7 @@ pub extern "C" fn rs_template_parse_request(
     _flags: u8,
 ) -> AppLayerResult {
     let eof = unsafe {
-        if AppLayerParserStateIssetFlag(pstate, APP_LAYER_PARSER_EOF) > 0 {
+        if AppLayerParserStateIssetFlag(pstate, APP_LAYER_PARSER_EOF_TS) > 0 {
             true
         } else {
             false
@@ -352,7 +353,7 @@ pub extern "C" fn rs_template_parse_response(
     _flags: u8,
 ) -> AppLayerResult {
     let _eof = unsafe {
-        if AppLayerParserStateIssetFlag(pstate, APP_LAYER_PARSER_EOF) > 0 {
+        if AppLayerParserStateIssetFlag(pstate, APP_LAYER_PARSER_EOF_TC) > 0 {
             true
         } else {
             false
