@@ -249,6 +249,13 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
             }
 
             jsb.set_string("server_guid", &guid_to_string(&x.server_guid))?;
+
+            if state.max_read_size > 0 {
+                jsb.set_uint("max_read_size", state.max_read_size.into())?;
+            }
+            if state.max_write_size > 0 {
+                jsb.set_uint("max_write_size", state.max_write_size.into())?;
+            }
         },
         Some(SMBTransactionTypeData::TREECONNECT(ref x)) => {
             jsb.set_uint("tree_id", x.tree_id as u64)?;
