@@ -889,15 +889,6 @@ error:
  */
 void IPOnlyInit(DetectEngineCtx *de_ctx, DetectEngineIPOnlyCtx *io_ctx)
 {
-    io_ctx->sig_init_size = DetectEngineGetMaxSigId(de_ctx) / 8 + 1;
-
-    if ( (io_ctx->sig_init_array = SCMalloc(io_ctx->sig_init_size)) == NULL) {
-        FatalError(SC_ERR_FATAL,
-                   "Fatal error encountered in IPOnlyInit. Exiting...");
-    }
-
-    memset(io_ctx->sig_init_array, 0, io_ctx->sig_init_size);
-
     io_ctx->tree_ipv4src = SCRadixCreateRadixTree(SigNumArrayFree,
                                                   SigNumArrayPrint);
     io_ctx->tree_ipv4dst = SCRadixCreateRadixTree(SigNumArrayFree,
@@ -965,10 +956,6 @@ void IPOnlyDeinit(DetectEngineCtx *de_ctx, DetectEngineIPOnlyCtx *io_ctx)
     if (io_ctx->tree_ipv6dst != NULL)
         SCRadixReleaseRadixTree(io_ctx->tree_ipv6dst);
     io_ctx->tree_ipv6dst = NULL;
-
-    if (io_ctx->sig_init_array)
-        SCFree(io_ctx->sig_init_array);
-    io_ctx->sig_init_array = NULL;
 }
 
 /**
@@ -1133,7 +1120,7 @@ void IPOnlyMatchPacket(ThreadVars *tv,
                             }
                         }
                     }
-                    PacketAlertAppend(det_ctx, s, p, 0, 0);
+                    AlertQueueAppend(det_ctx, s, p, 0, 0);
                 }
             }
         }
@@ -1142,7 +1129,7 @@ void IPOnlyMatchPacket(ThreadVars *tv,
 }
 
 /**
- * \brief Build the radix trees from the lists of parsed adresses in CIDR format
+ * \brief Build the radix trees from the lists of parsed addresses in CIDR format
  *        the result should be 4 radix trees: src/dst ipv4 and src/dst ipv6
  *        holding SigNumArrays, each of them with a hierarchical relation
  *        of subnets and hosts
@@ -1562,7 +1549,7 @@ void IPOnlyPrepare(DetectEngineCtx *de_ctx)
 
 /**
  * \brief Add a signature to the lists of Addresses in CIDR format (sorted)
- *        this step is necesary to build the radix tree with a hierarchical
+ *        this step is necessary to build the radix tree with a hierarchical
  *        relation between nodes
  * \param de_ctx Pointer to the current detection engine context
  * \param de_ctx Pointer to the current ip only detection engine contest
@@ -1589,9 +1576,6 @@ void IPOnlyAddSignature(DetectEngineCtx *de_ctx, DetectEngineIPOnlyCtx *io_ctx,
     if (s->num > io_ctx->max_idx)
         io_ctx->max_idx = s->num;
 
-    /* enable the sig in the bitarray */
-    io_ctx->sig_init_array[(s->num/8)] |= 1 << (s->num % 8);
-
     /** no longer ref to this, it's in the table now */
     s->CidrSrc = NULL;
     s->CidrDst = NULL;
@@ -1619,7 +1603,7 @@ static int IPOnlyTestSig01(void)
 }
 
 /**
- * \test check that we dont set a Signature as IPOnly because it has no rule
+ * \test check that we don't set a Signature as IPOnly because it has no rule
  *       option appending a SigMatch but a port is fixed
  */
 
@@ -1639,7 +1623,7 @@ static int IPOnlyTestSig02 (void)
 }
 
 /**
- * \test check that we set dont set a Signature as IPOnly
+ * \test check that we set don't set a Signature as IPOnly
  *  because it has rule options appending a SigMatch like content, and pcre
  */
 
